# coding: utf-8

#Flask
from flask import render_template, Blueprint, request, current_app

# Subprocessing for restarting supervisor
from modislock_webservice.tasks import send_async_command

# Form
from forms.welcome import WelcomeForm

# Security
from flask_security.utils import hash_password

# Database
from models import db, User, Settings, SettingsValues
from sqlalchemy.exc import IntegrityError, InternalError

# Date time
from datetime import datetime
from modislock_webservice.utils.sys_rtc import ntp_clock_sync, time_zone, system_time
from modislock_webservice.utils.sys_ssh import system_ssh

bp = Blueprint('welcome', __name__)


def save_reg_data(form):
    """Save registration data to database

    :param form:
    :return:
    """
    # Serial Number
    user = User.query.filter_by(id=1).one_or_none()
    serial = Settings.query.filter(Settings.settings_name == 'SERIAL_NUMBER').one_or_none()
    registration = Settings.query.filter(Settings.settings_name == 'REGISTRATION').one_or_none()

    if serial is not None and registration is not None and user is not None:
        user.first_name = form.first_name.data
        user.last_name = form.last_name.data
        user.email = form.email.data
        user.password = hash_password(form.password.data)

        serial_num = SettingsValues.query\
            .filter(SettingsValues.settings_id_settings == serial.id_settings).one_or_none()
        reg_num = SettingsValues.query \
            .filter(SettingsValues.settings_id_settings == registration.id_settings).one_or_none()

        if serial_num is not None:
            serial_num.value = form.serial_number.data

        if reg_num is not None:
            reg_num.value = 'DISABLED'

        try:
            db.session.commit()
        except (IntegrityError, InternalError):
            current_app.logger.debug('Error writing to database')
            db.session.rollback()

    # Timezone and Time
    if time_zone() != form.tz_zone.data:
        time_zone(form.tz_zone.data)

    if form.auto_time.data != ntp_clock_sync():
        ntp_clock_sync(enabled=form.auto_time.data)

    if not form.auto_time.data:
        try:
            system_time(form.sys_time.data)
        except TypeError:
            current_app.logger.debug('Wrong format for system setting')

    if form.ssh_enable.data != system_ssh():
        system_ssh(form.ssh_enable.data)

    return True


@bp.route('/', methods=['GET', 'POST'])
def welcome():
    """Registration Page

    Controls the initial registration window(s).
    1. Serial Number
    2. Admin email address
    3. Admin password
    4. System Time Date Timezone
    5. Timezone

    :returns html_template welcome.html:
    """
    form = WelcomeForm()

    if request.method == 'POST':
        if form.validate():
            if save_reg_data(form):
                args = ['supervisorctl', 'restart', 'admin:modis_admin']
                try:
                    send_async_command.delay(args, countdown=3)
                except:
                    pass
                return render_template('welcome/birthing_new_lock.html', domain=current_app.config.get('SITE_DOMAIN'))
        else:
            current_app.logger.error('Could not validate registration form')

    form.tz_zone.data = time_zone()
    form.sys_time.data = datetime.now().strftime('%H:%M:%S %Y-%m-%d')
    form.auto_time.data = ntp_clock_sync()
    form.ssh_enable.data = system_ssh()

    return render_template('welcome/welcome.html', form=form)
